@@ -63,6 +63,10 @@ char *strdup();
 #include <paths.h>
 #endif
 
+#ifndef MAXHOSTNAMELEN
+#include <netdb.h>	/* SunOS 5.x... */
+#endif
+
 #ifndef NO_TTYENT
 #include <ttyent.h>
 #endif
@@ -106,13 +110,14 @@ main(argc, argv)
 	uid_t ruid, getuid();
 	int asme, ch, asthem, fastlogin, prio;
 	enum { UNSET, YES, NO } iscsh = UNSET;
-	char *user, *shell, *username, *nargv[4], **np;
+	char *user, *shell, *username, *nargv[6], **np;
 	char shellbuf[MAXPATHLEN];
 	char *crypt(), *getpass(), *getenv(), *getlogin(), *ontty();
 #ifdef SKEY
 	char *skey_crypt(), *skey_getpass();
 	int pwok = 1;
 #endif
+	extern char *optarg;
 
 	np = &nargv[3];
 	*np-- = NULL;
@@ -186,14 +191,17 @@ main(argc, argv)
 		fprintf(stderr, "su: unknown login %s\n", user);
 		exit(1);
 	}
+	if (*argv && (nargv[3] = argv[1]) && (nargv[4] = argv[2]))
+		nargv[5] = 0;
 
 	if (ruid) {
 #ifdef KERBEROS
 	    if (!use_kerberos || kerberos(username, user, pwd->pw_uid))
 #endif
 	    {
+#ifndef NO_WHEEL_GROUP
 		/* only allow those in group zero to su to root. */
-		if (pwd->pw_uid == 0 && (gr = getgrgid((gid_t)0)))
+		if (pwd->pw_uid == 0 && (gr = getgrgid(0)) && gr->gr_mem) {
 			for (g = gr->gr_mem;; ++g) {
 				if (!*g) {
 					(void)fprintf(stderr,
@@ -207,7 +215,9 @@ main(argc, argv)
 				}
 				if (!strcmp(username, *g))
 					break;
+			}
 		}
+#endif /* NO_WHEEL_GROUP */
 		/* if target requires a password, verify it */
 		if (*pwd->pw_passwd) {
 #ifdef SKEY
@@ -297,8 +307,12 @@ main(argc, argv)
 				exit(1);
 			}
 		}
-		if (asthem || pwd->pw_uid)
+		if (asthem || pwd->pw_uid) {
+#ifdef ENV_LOGNAME
+			(void)setenv("LOGNAME", pwd->pw_name, 1);
+#endif
 			(void)setenv("USER", pwd->pw_name, 1);
+		}
 		(void)setenv("HOME", pwd->pw_dir, 1);
 		(void)setenv("SHELL", shell, 1);
 	}
@@ -346,12 +360,16 @@ chshell(sh)
 char *
 ontty()
 {
-	char *p, *ttyname();
-	static char buf[MAXPATHLEN + 4];
+	char *p, *ttyname(), *utmp_host();
+	static char buf[MAXPATHLEN + 4 + MAXHOSTNAMELEN + 6];
 
 	buf[0] = 0;
 	if (p = ttyname(STDERR_FILENO))
 		sprintf(buf, " on %s", p);
+	if (p = utmp_host()) {
+		strcat(buf, " from ");
+		strcat(buf, p);
+	}
 	return (buf);
 }
 
