@@ -8,8 +8,10 @@
 #include <sys/resource.h>
 
 /* 44BSD compatibility. */
-#ifdef RLIMIT_OFILE
-#define RLIMIT_NOFILE RLIMIT_OFILE
+#ifndef RLIMIT_NOFILE
+# ifdef RLIMIT_OFILE
+#  define RLIMIT_NOFILE RLIMIT_OFILE
+# endif
 #endif
 
 /* open_limit - determine the current open file limit */
