@@ -1,10 +1,16 @@
 /* Author: Wietse Venema <wietse@wzv.win.tue.nl> */
 
+#include <stdio.h>
 #include <string.h>
 #include <pwd.h>
 #ifndef NO_ULIMIT_H
 #include <ulimit.h>
 #endif
+#ifdef HAS_PATHS_H
+#include <paths.h>
+#else
+#include "paths.h"
+#endif
 
 #include "sys_defs.h"
 
@@ -122,7 +128,11 @@ char   *term;
 
     /* PATH is always reset. */
 
+#ifdef SYSV_LOGINDEFS
     setenv("PATH", pwd->pw_uid ? default_path : default_supath, 1);
+#else
+    setenv("PATH", _PATH_DEFPATH, 1);
+#endif
 
     /* Undocumented: HOME, MAIL and LOGNAME are always reset (SunOS 5.1). */
 
@@ -141,7 +151,7 @@ char   *term;
      * Extension: when ALTSHELL=YES, we set the SHELL variable even if it is
      * /bin/sh.
      */
-
+#ifdef SYSV_LOGINDEFS
     if (strcasecmp(default_altsh, "YES") == 0)
 	setenv("SHELL", pwd->pw_shell, 1);
     if (default_hz)
@@ -159,4 +169,5 @@ char   *term;
 	if (sscanf(default_ulimit, "%ld", &limit_val) == 1 && limit_val)
 	    ulimit(UL_SETFSIZE, limit_val);
     }
+#endif /* SYSV_LOGINDEFS */
 }
