@@ -169,7 +169,7 @@ main(argc, argv)
 	fromlen = sizeof (from);
 	if (getpeername(0, (struct sockaddr *) &from, &fromlen) < 0) {
 		syslog(LOG_ERR, "Couldn't get peer name of remote host: %m");
-		fatalperror("Can't get peer name of host");
+		fatalperror(0, "Can't get peer name of host");
 	}
 	if (keepalive &&
 	    setsockopt(0, SOL_SOCKET, SO_KEEPALIVE, (char *) &on, sizeof (on)) < 0)
@@ -184,6 +184,12 @@ extern	char	*inet_ntoa();
 
 struct winsize win = { 0, 0, 0, 0 };
 
+#ifdef REQUEST_INFO_DECLARED
+#define request_info tcpd_request_info
+#endif
+
+#include <tcpd.h>
+
 
 doit(f, fromp)
 	int f;
@@ -195,6 +201,9 @@ doit(f, fromp)
 	register struct hostent *hp;
 	char c;
 	struct termios termios;
+	struct request_info request;
+	struct sockaddr_in me;
+	int me_len;
 
 	alarm(60);
 	read(f, &c, 1);
@@ -274,8 +283,21 @@ doit(f, fromp)
 		    write(f, "rlogind: Host address mismatch.\r\n",
 		     sizeof("rlogind: Host address mismatch.\r\n") - 1);
 	}
-	do_access(procname, remotehost, inet_ntoa(fromp->sin_addr), 
-		  rusername, lusername);
+	me_len = sizeof(me);
+	if (getsockname(f, (struct sockaddr *) &me, &me_len) < 0) {
+		syslog(LOG_ERR, "getsockname: %m");
+		exit(1);
+	}
+	request_init(&request,
+		     RQ_DAEMON, procname,
+		     RQ_CLIENT_NAME, NOT_INADDR(remotehost) ? remotehost : "unknown",
+		     RQ_CLIENT_SIN, fromp,
+		     RQ_USER, rusername,
+		     RQ_FILE, f,
+		     RQ_SERVER_SIN, &me,
+		     0);
+	sock_methods(&request);
+	do_access(&request, remotehost, rusername, lusername);
 
 #ifdef STREAM_PTY
 	if ((p = open("/dev/ptmx", O_RDWR)) < 0)
@@ -746,27 +768,12 @@ setup_term(tp)
 int     allow_severity = LOG_INFO;
 int     deny_severity = LOG_WARNING;
 
-#ifdef REQUEST_INFO_DECLARED
-#define request_info tcpd_request_info
-#endif
-
-#include <tcpd.h>
-
-do_access(daemon, host, addr, ruser, luser)
-char   *daemon;
-char   *host;
-char   *addr;
+do_access(request, host, ruser, luser)
+struct request_info *request;
 char   *ruser;
 char   *luser;
 {
-    struct request_info request;
-    int     allow = hosts_access(request_init(&request,
-					      RQ_DAEMON, daemon,
-		      RQ_CLIENT_NAME, strcmp(host, addr) ? host : "unknown",
-					      RQ_CLIENT_ADDR, addr,
-					      RQ_USER, ruser,
-					      RQ_FILE, 0,
-					      0)) && (luser[0] != '-');
+    int     allow = hosts_access(request) && (luser[0] != '-');
     char   *text = allow ? "" : "refused ";
     int     severity = allow ? allow_severity : deny_severity;
 
