@@ -152,6 +152,8 @@ char	path[] = "PATH=/usr/ucb:/bin:/usr/bin:";
 char	**environ;
 #endif
 
+#include <tcpd.h>
+
 doit(fromp)
 	struct sockaddr_in *fromp;
 {
@@ -167,6 +169,9 @@ doit(fromp)
 	char buf[BUFSIZ], sig;
 	int one = 1;
 	char remotehost[2 * MAXHOSTNAMELEN + 1];
+	struct request_info request;
+	struct sockaddr_in me;
+	int me_len;
 
 	(void) signal(SIGINT, SIG_DFL);
 	(void) signal(SIGQUIT, SIG_DFL);
@@ -301,8 +306,21 @@ doit(fromp)
 	getstr(remuser, sizeof(remuser), "remuser");
 	getstr(locuser, sizeof(locuser), "locuser");
 	getstr(cmdbuf, sizeof(cmdbuf), "command");
-	do_access(procname, remotehost, inet_ntoa(fromp->sin_addr), 
-		remuser, locuser, cmdbuf);
+        me_len = sizeof(me);
+        if (getsockname(0, (struct sockaddr *) &me, &me_len) < 0) {
+                syslog(LOG_ERR, "getsockname: %m");
+                exit(1);
+        }
+        request_init(&request,
+                     RQ_DAEMON, procname,
+                     RQ_CLIENT_NAME, NOT_INADDR(remotehost) ? remotehost : "unknown",
+                     RQ_CLIENT_SIN, fromp,
+                     RQ_USER, remuser,
+                     RQ_FILE, 0,
+                     RQ_SERVER_SIN, &me,
+                     0);
+        sock_methods(&request);
+	do_access(&request, remotehost, remuser, locuser, cmdbuf);
 	setpwent();
 	pwd = getpwnam(locuser);
 	if (pwd == NULL) {
@@ -444,27 +462,17 @@ getstr(buf, cnt, err)
 	} while (c != 0);
 }
 
-#include <tcpd.h>
-
 int     allow_severity = LOG_INFO;
 int     deny_severity = LOG_WARNING;
 
-do_access(daemon, host, addr, ruser, luser, command)
-char   *daemon;
+do_access(request, host, ruser, luser, command)
+struct request_info *request;
 char   *host;
-char   *addr;
 char   *ruser;
 char   *luser;
 char   *command;
 {
-    struct request_info request;
-    int     allow = hosts_access(request_init(&request,
-					      RQ_DAEMON, daemon,
-		      RQ_CLIENT_NAME, strcmp(host, addr) ? host : "unknown",
-					      RQ_CLIENT_ADDR, addr,
-					      RQ_USER, ruser,
-					      RQ_FILE, 0,
-					      0));
+    int     allow = hosts_access(request);
     char   *text = allow ? "" : "refused ";
     int     severity = allow ? allow_severity : deny_severity;
 
